
(defun my-progress-org-mode-ask-effort ()
  "Ask for an effort estimate when clocking in.  From http://nflath.com/2010/03/"
  (unless (org-entry-get (point) "Effort") 
    (let ((effort
           (completing-read
            "Effort: "
            (org-entry-get-multivalued-property (point) "Effort"))))
      (unless (equal effort "")
        (org-set-property "Effort" effort)))))

(provide 'my-progress)

(defun toc-alist ()
  (let (
        (ascii (org-export-as 'my-progress-toc))
        (mylist nil)
        (newcons ""))
    (dolist (bb (split-string ascii "\n"))
      (when (string-match "^\\(\.*\\(  \\)*[.1-9]+\\) \\([a-z].*\\)$" bb)
        (setq newcons (cons
                       (match-string 3 bb)
                       (match-string 1 bb)))
        (add-to-list 'mylist newcons)))
    mylist))

(defun org-dblock-write:columnview-toc (params)
    "Write the column view table.
  PARAMS is a property list of parameters:

  :width    enforce same column widths with <N> specifiers.
  :id       the :ID: property of the entry where the columns view
            should be built.  When the symbol `local', call locally.
            When `global' call column view with the cursor at the beginning
            of the buffer (usually this means that the whole buffer switches
            to column view).  When \"file:path/to/file.org\", invoke column
            view at the start of that file.  Otherwise, the ID is located
            using `org-id-find'.
  :hlines   When t, insert a hline before each item.  When a number, insert
            a hline before each level <= that number.
  :vlines   When t, make each column a colgroup to enforce vertical lines.
  :maxlevel When set to a number, don't capture headlines below this level.
  :skip-empty-rows
            When t, skip rows where all specifiers other than ITEM are empty."
    (require 'org-timer)
    (let ((pos (move-marker (make-marker) (point)))
          (hlines (plist-get params :hlines))
          (vlines (plist-get params :vlines))
          (maxlevel (plist-get params :maxlevel))
          (content-lines (org-split-string (plist-get params :content) "\n"))
          (skip-empty-rows (plist-get params :skip-empty-rows))
          tbl id idpos nfields tmp recalc line
          id-as-string view-file view-pos mytoc)
       (when (setq id (plist-get params :id))
        (setq id-as-string (cond ((numberp id) (number-to-string id))
                                 ((symbolp id) (symbol-name id))
                                 ((stringp id) id)
                                 (t "")))
        (cond ((not id) nil)
              ((eq id 'global) (setq view-pos (point-min)))
              ((eq id 'local))
              ((string-match "^file:\\(.*\\)" id-as-string)
              (setq view-file (match-string 1 id-as-string)
                     view-pos 1)
               (unless (file-exists-p view-file)
                 (error "No such file: \"%s\"" id-as-string)))
              ((setq idpos (org-find-entry-with-id id))
               (setq view-pos idpos))
              ((setq idpos (org-id-find id))
               (setq view-file (car idpos))
               (setq view-pos (cdr idpos)))
              (t (error "Cannot find entry with :ID: %s" id))))
      (with-current-buffer (if view-file
                               (get-file-buffer view-file)
                             (current-buffer))
        (save-excursion (setq mytoc (reverse (toc-alist))))
        (save-excursion
          (save-restriction
            (widen)
            (goto-char (or view-pos (point)))
            (org-columns)
            (setq tbl (org-columns-capture-view maxlevel skip-empty-rows))
            (setq nfields (length (car tbl)))
            (org-columns-quit))))
      (goto-char pos)
      (move-marker pos nil)
      (when tbl
        (when (plist-get params :hlines)
          (setq tmp nil)
          (while tbl
            (if (eq (car tbl) 'hline)
                (push (pop tbl) tmp)
              (if (string-match "\\` *\\(\\*+\\)" (caar tbl))
                  (if (and (not (eq (car tmp) 'hline))
                           (or (eq hlines t)
                               (and (numberp hlines)
                                    (<= (- (match-end 1) (match-beginning 1))
                                        hlines))))
                      (push 'hline tmp)))
              (push (pop tbl) tmp)))
          (setq tbl (nreverse tmp)))
        (when vlines
          (setq tbl (mapcar (lambda (x)
                              (if (eq 'hline x) x (cons "" x)))
                            tbl))
          (setq tbl (append tbl (list (cons "/" (make-list nfields "<>"))))))
        (setq pos (point))
        (when content-lines
          (while (string-match "^#" (car content-lines))
            (insert (pop content-lines) "\n")))
        (setq tmptbl (org-listtable-to-string tbl))
        ;; swap half of the line for the numbers in the TOC
        (let ((shline "")
              (keya "")
              (nline "")
              (lenline 1)
              (task "")
             (was-dash nil)
             (first-char ".")
             (accum-time 0))
          (dolist (shline (split-string tmptbl "\n"))
          (let ((is-dash (string= "|-|" shline))
                (writeline t))
            (if (and (string-match "^|\\([*]*\\) \\(.*\\)" shline)
                     (> (length (match-string 1 shline)) 12))
                (progn (setq task (match-string 2 shline))
                       (when (string-match "\\(TODO \\|NEXT \\)" task)
                         (setq task (replace-match "" nil t task 0))
                         (setq shline (concat "|"
                                              first-char
                                              (make-string lenline ?\s) task "|")))
                       (when (string-match "DONE " task)
                         (setq shline "DEL")))
              ;; if it is a first level heading then accumulate the time value
              ;; e.g. "|* XX |49:30:2|"
              (when (string-match "^|[*][ ]+.*|\\([0-9]*:*[0-9]*:*[0-9]+\\)|" shline)
                (let ((str-time (match-string 1 shline)))
                  (when str-time               ;; found a time
                    (setq accum-time
                          (+ accum-time
                             (org-timer-hms-to-secs
                              (org-timer-fix-incomplete
                               ;; assume times are HH:MM and so append :00 seconds
                               (concat (match-string 1 shline) ":00"))))))))
              ;; if it is a headline, add the section number to the start
              (when (string-match "^|\\(.*[*] \\)\\(.*?\\)[ ]*|.*" shline)
                (setq keya (match-string 2 shline))
                (setq toc-num (cdr (assoc-string keya mytoc)))
                ;; if the heading had an entry in the toc then include the
                ;; heading and replace with the toc section number, otherwise
                ;; don't include the line at all, this will effectively exclude
                ;; headings not for export
                (if toc-num
                    (progn
                      (setq mstr (concat first-char toc-num " "))
                      (message (format "shline: %s keya: %s mstr: %s" shline keya mstr))
                      (setq lenline (length mstr))
                      (setq shline (replace-match mstr nil t shline 1))
                      (setq mytoc (remove (assoc-string keya mytoc) mytoc )))
                  (setq shline "DEL"))))
            ;; add back in
            (if (string-match "\\(^|[*]* END.*\\|^DEL\\)" shline)
                (progn
                  (message
                   (concat " org-dblock-write:columnview-toc   : Excluding"
                           shline))
                  (setq writeline nil)))
            ;; don't want two dashed rows next to each other
            (if is-dash
                (progn
                  (when was-dash
                    (setq writeline nil))
                  (setq was-dash t))
              ;; if writeline is false then leave was-dash untouched
              (when writeline (setq was-dash nil)))
            (when writeline
              (setq nline (concat nline shline "\n")))))
        ;; add a row for the the accumulated time
        ;; how many rows?
        (setq accum-time-hms (org-timer-secs-to-hms accum-time))
        (string-match "^\\([0-9]+:[0-9]+\\)\\(:[0-9]+\\)" accum-time-hms)
        (message accum-time-hms)
        (setq accum-time-hm (match-string 1 accum-time-hms))
        ;;(match-string 1 shline)
        (setq nline (concat
                     nline
;;                   "|Total time: |" (org-timer-secs-to-hms accum-time) "|\n"))
                     "|-|-|\n|Total time [H:M]: |" accum-time-hm "|\n"))
        (setq tmptbl nline))
        ;; ----------- (end swap) ---------
        (insert tmptbl)
        (when (plist-get params :width)
          (insert "\n|" (mapconcat (lambda (x) (format "<%d>" (max 3 x)))
                                   org-columns-current-widths "|")))
        (while (setq line (pop content-lines))
          (when (string-match "^#" line)
            (insert "\n" line)
            (when (string-match "^[ \t]*#\\+TBLFM" line)
              (setq recalc t))))
        (if recalc
            (progn (goto-char pos) (org-table-recalculate 'all))
          (goto-char pos)
          (org-table-align)))))

(defun my-progress-ascii-toc-template (contents info)
  "Return complete document string after ASCII conversion.
CONTENTS is the transcoded contents string.  INFO is a plist
holding export options.
Cookie cut from org-e-ascii-template.
"
  (org-element-normalize-string
   (org-e-ascii--indent-string
    (let ((text-width (- org-e-ascii-text-width org-e-ascii-global-margin)))
      (concat
       (let ((depth (plist-get info :with-toc)))
         (when depth
           (concat
            (org-e-ascii--build-toc info (and (wholenump depth) depth))
            "\n\n\n")))))
    org-e-ascii-global-margin)))

(require 'org-e-ascii)
(org-export-define-derived-backend my-progress-toc e-ascii
  :translate-alist ((template . my-progress-ascii-toc-template)))

(defun my-progress-remove-toc-heading (output backend info)
  (when (and (memq backend '(my-progress-toc))
             (string-match
              "Table of Contents\n_________________\n\n"
              output))
    (replace-match "" nil nil output)))

;;(add-to-list 'org-export-filter-final-output-functions 'my-progress-remove-toc-heading)
